EBB_BarGroup = AceLibrary("AceOO-2.0").Class()

function EBB_BarGroup.prototype:Init2()
	self.bars = {}
	self.data = {}
	self.frames = {}
	self.layout = {}
	self.layoutbars = {}
	if not self.frames.container then
		self.frames.container = CreateFrame("button", nil, UIParent)
		self.frames.container:SetFrameStrata("BACKGROUND")
		self.frames.container:SetMovable(true)
	end
	self.anchorshown = false
end

function EBB_BarGroup.prototype:GetContainer()
	return self.frames.container
end

function EBB_BarGroup.prototype:SetLayout(layout, layoutbars)
	self.layout = layout
	self.layoutbars = layoutbars
	for i = 1, #self.bars do
		self.bars[i]:UpdateLayout(layoutbars)
		slef.bars[i]:UpdateData()
	end
	self:UpdateBarPositions()
end

function EBB_BarGroup.prototype:ToggleAnchor(show)
	if show then
		if not self.frames.anchor then
			self.frames.anchor = CreateFrame("Button", nil, self.frames.container)
			self.frames.anchor:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
											edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
											tile = false, tileSize = 16, edgeSize = 16,
											insets = { left = 5, right =5, top = 5, bottom = 5 }})
			self.frames.anchor:SetBackdropColor(self.layout.anchorcolor[1], self.layout.anchorcolor[2], self.layout.anchorcolor[3], .5)
			self.frames.anchor:SetHeight(20)
			self.frames.anchor:SetWidth(self.layoutbars.width)
			self.frames.anchortext = self.frames.anchor:CreateFontString(nil, "OVERLAY")
			self.frames.anchortext:SetFontObject(GameFontNormalSmall)
			self.frames.anchortext:ClearAllPoints()
			self.frames.anchortext:SetTextColor(1, 1, 1, 1)
			self.frames.anchortext:SetPoint("TOP", self.frames.anchor, "TOP")
			self.frames.anchortext:SetJustifyH("CENTER")
			self.frames.anchortext:SetJustifyV("MIDDLE")
			self.frames.anchortext:SetText(self.layout.anchortext)

			self.frames.anchor:RegisterForDrag("LeftButton")
			self.frames.anchor:SetScript("OnDragStart", function() this:GetParent():StartMoving() end )
			self.frames.anchor:SetScript("OnDragStop", function() this:GetParent():StopMovingOrSizing() end )
		end
		self.frames.anchor:Show()
		self.frames.container:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
											edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
											tile = false, tileSize = 16, edgeSize = 16,
											insets = { left = 5, right =5, top = 5, bottom = 5 }})
		self.anchorshown = true
	else
		if self.frames.anchor then
			self.frames.anchor:Hide()
		end
		self.frames.container:SetBackdrop(nil)
		self.anchorshown = false
	end
	self:UpdateBarPositions()
end

-- updates position of bars (+ anchor) inside the container; sets container height
function EBB_BarGroup.prototype:UpdateBarPositions()
	local lastframe
	local height = 0
	if self.anchorshown then
		self:UpdateBarPosition(self.frames.anchor, lastframe)
		lastframe = self.frames.anchor
		height = height + 20
	end
	for i = 1, #self.bars do
		if height > 0 then height = height + self.layout.barspacing end
		self:UpdateBarPosition(self.bars[i]:GetContainer(), lastframe)
		height = height + self.layoutbars.height
		lastframe = self.bars[i]:GetContainer()
	end
	height = height + 10
	self.frames.container:SetHeight(height)
	self.frames.container:SetWidth(self.layoutbars.width + 10)
end

function EBB_BarGroup.prototype:UpdateBarPosition(frame, relframe)
	if not relframe then
		frame:SetPoint(self.layout.growup and "BOTTOM" or "TOP", self.frames.container, self.layout.growup and "BOTTOM" or "TOP", 0, self.layout.growup and 5 or -5)
	else
		frame:SetPoint(self.layout.growup and "BOTTOM" or "TOP", relframe, self.layout.growup and "TOP" or "BOTTOM", 0, self.layout.barspacing)
	end
end

-- creates data from which the bars will be created; set external
function EBB_BarGroup.prototype:UpdateData()
	self.data = {}
	self:UpdateBars()
end

-- creates bars from data
function EBB_BarGroup.prototype:UpdateBars()
	i = 1
	while i <= #self.data do
		if i > #self.bars then
			table.insert(self.bars, ElkBuffBars:GetBar())
			self.bars[i]:UpdateLayout(self.layoutbars)
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] container: "..self.bars[i].frames.container:GetFrameStrata().." "..self.bars[i].frames.container:GetFrameLevel())
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] bgbar: "..self.bars[i].frames.bgbar:GetFrameStrata().." "..self.bars[i].frames.bgbar:GetFrameLevel())
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] bar: "..self.bars[i].frames.bar:GetFrameStrata().." "..self.bars[i].frames.bar:GetFrameLevel())
			self.bars[i]:GetContainer():SetParent(self.frames.container)
			self.bars[i]:GetContainer():SetFrameLevel(self.bars[i].frames.bar:GetFrameLevel() + 1) -- fix frame order
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] container: "..self.bars[i].frames.container:GetFrameStrata().." "..self.bars[i].frames.container:GetFrameLevel())
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] bgbar: "..self.bars[i].frames.bgbar:GetFrameStrata().." "..self.bars[i].frames.bgbar:GetFrameLevel())
			DEFAULT_CHAT_FRAME:AddMessage("[EBB] bar: "..self.bars[i].frames.bar:GetFrameStrata().." "..self.bars[i].frames.bar:GetFrameLevel())
		end
		self.bars[i]:UpdateData(self.data[i])
		self.bars[i]:GetContainer():Show()
		i = i + 1
	end
	while i <= #self.bars do
		ElkBuffBars:RecycleBar(table.remove(self.bars, #self.bars))
	end
	self:UpdateBarPositions()
end
